// ---------------------------------------------------------------------------------------

// Gira Funkbus Events by EL
// Software to capture funkbus events
// Layer on receiving en sending telegrams using EL_FunkBus
// Version 01-05-2024

// ---------------------------------------------------------------------------------------

#pragma once

#ifndef funkBusEvents_H 
#define funkBusEvents_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include "esp_timer.h"

// ---------------------------------------------------------------------------------------
// FunkBus events is based on EL_FunkBus
// ---------------------------------------------------------------------------------------

#include "EL_FunkBus.hpp"

// ---------------------------------------------------------------------------------------
// Examples to use this lib
// ---------------------------------------------------------------------------------------

/*
#define FUNKBUS_SEND_PIN        (GPIO_NUM_6)   // pin6 ESP-S3-Zero
#define FUNKBUS_RECEIVE_PIN     (GPIO_NUM_5)   // pin5 ESP-S3-Zero

initFunkBusEvents(FUNKBUS_RECEIVE_PIN, FUNKBUS_SEND_PIN, funkBusReceiveEventsCallBackHandler);

void funkBusReceiveEventsCallBackHandler(const funkBusEvent * event)
{
    // show in monitor
    printFunkBusEvent(TAG, "Receive event", event);
    // send the received event on the bus to make the inserted module apear (almost) transparrant
    sendFunkBusEvent(event);
    // Process only the events of intrest 
    if ((event->event == fbDimMore) || (event->event == fbDimLess) || (event->event == fbSwitchOn) || (event->event == fbSwitchOff))
    {
        funkBusButton fb = event->button;
    }
}

*/

// ---------------------------------------------------------------------------------------
// FunkBusEvents specific structs
// ---------------------------------------------------------------------------------------

enum funkBusEventType
{
    fbNoEvent = 0,
    fbSwitchOn,
    fbSwitchOff,
    fbDimMore,
    fbDimLess,
    fbDimMoreRepeat,
    fbDimLessRepeat,
    fbDimEnd
};

typedef struct
{
    funkBusEventType event;
    funkBusButton button;
    funkBusChannel channel;
} funkBusEvent;

// ---------------------------------------------------------------------------------------
// FunkBus callback functiontypes
// ---------------------------------------------------------------------------------------

typedef void (*funkBusEventCallBack)(const funkBusEvent * event);

// ---------------------------------------------------------------------------------------
// Public functions
// ---------------------------------------------------------------------------------------

// Lower level and library function control

void initFunkBusEvents(const int receivePin, const int sendPin, const funkBusEventCallBack eventCb);

bool sendFunkBusEvent(const funkBusEvent * event);

bool funkBusDimStep(const bool more, const funkBusButton * button, const funkBusChannel * channel);

bool funkBusSwitch(const bool on, const funkBusButton * button, const funkBusChannel * channel);

char * funkBusEventTypeToStr(const funkBusEventType eventType);

void printFunkBusEvent(const char * tag, const char * title, const funkBusEvent * event);

// ---------------------------------------------------------------------------------------
// End
// ---------------------------------------------------------------------------------------

#ifdef __cplusplus
} /*extern "C"*/
#endif

#endif /*funkBusEvents_H*/


